package com.docus.server.visitor;

import com.docus.core.util.Func;
import com.docus.core.util.json.JSON;
import com.docus.log.context.TrackContext;
import com.docus.log.processor.AbstractProcessor;
import com.docus.server.collect.infrastructure.dao.CollectTypeEnum;
import com.docus.server.collect.infrastructure.dao.StateEnum;
import com.docus.server.collect.infrastructure.enums.IIntegerEnum;
import com.docus.server.tool.ParamsUtils;
import com.docus.server.tool.SpringUtils;
import com.docus.server.ws.ITaskOriginalMessageService;
import com.docus.server.ws.IWsResult;
import com.docus.server.ws.convert.IConverter;
import lombok.extern.slf4j.Slf4j;

import java.util.Map;

/**
 * @author linruifeng
 */
@Slf4j
public class VisitorProcessor extends AbstractProcessor {
    private ITaskOriginalMessageService messageService;
    private IConverter converter;
    private IWsResult wsResult;

    /**
     * 前置通知
     */
    @Override
    public Object beforeProcess(TrackContext context) {
        super.beforeProcess(context);
        initBeans(context.getBeanNames());
        String message = (String) context.getArgs()[0];
        if (Func.isEmpty(message)) {
            throw new RuntimeException("参数为空");
        }
        String jsonStr = JSON.toJSON(converter.convert(message, context.getMethodName()));
        Long taskId = messageService.insertTaskOriginalMessage(jsonStr, message,
                IIntegerEnum.fromDisplay(CollectTypeEnum.class, context.getGroup()));
        context.setParams(ParamsUtils.addParam("taskId", taskId).addParam("jsonStr", jsonStr).param());
        return null;
    }

    /**
     * 后置通知
     */
    @Override
    public Object afterReturnProcess(TrackContext context) {
        Map<String, Object> params = context.getParams();
        Long taskId = (Long) params.get("taskId");
        String afterReturnResult = (String) params.get("jsonStr");
        messageService.updateTaskOriginalMessage(taskId, afterReturnResult, StateEnum.OK);
        Map map = JSON.fromJSON(afterReturnResult, Map.class);
        map.put("msg", "操作成功！");
        return wsResult.ok(map);
    }

    /**
     * 异常通知
     */
    @Override
    public Object afterThrowingProcess(TrackContext context) {
        log.error(context.getExMessageResult());
        Map<String, Object> params = context.getParams();
        try {
            params.put("msg", context.getExMessageResult());
            Map<String, Object> beforeResult = context.getParams();
            Long taskId = (Long) beforeResult.get("taskId");
            String afterReturnResult = (String) beforeResult.get("jsonStr");
            messageService.updateTaskOriginalMessage(taskId, afterReturnResult, StateEnum.FAIL);
            params.putAll(JSON.fromJSON(afterReturnResult, Map.class));
            return wsResult.fail(params);
        } catch (Exception e) {
            return wsResult.fail(params);
        }
    }

    private void initBeans(String[] beanNames) {
        this.converter = (IConverter) SpringUtils.getBean(beanNames[0]);
        this.wsResult = (IWsResult) SpringUtils.getBean(beanNames[1]);
        this.messageService = SpringUtils.getBean(ITaskOriginalMessageService.class);
    }
}
