package com.docus.log.processor;

import com.docus.log.context.TrackContext;
import com.docus.log.context.TrackHelper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public abstract class AbstractProcessor implements ITrackProcessor {

    private final Logger logger = LoggerFactory.getLogger(AbstractProcessor.class);

    public boolean validate(TrackContext context) {
        return context != null
                && context.getGroup() != null
                && context.getProcessor() != null;
    }

    /**
     * 前置通知
     */
    @Override
    public Object beforeProcess(TrackContext context) {
        logger.debug("=== AOP 前置通知 ===");
        return null;
    }

    /**
     * 后置通知和异常通知
     */
    @Override
    public Object process(TrackContext context) {
        if (validate(context)) {
            Object o = doProcess(context);
            TrackHelper.setParams(context.getParams());
            return o;
        }
        return null;
    }

    /**
     * 最后通知
     */
    @Override
    public Object afterProcess(TrackContext context) {
        logger.debug("=== AOP 最后通知 ===");
        return null;
    }

    public Object doProcess(TrackContext context) {
        if (context.isError()) {
            logger.debug("=== AOP 异常通知 ===");
            return afterThrowingProcess(context);
        } else {
            logger.debug("=== AOP 后置通知 ===");
            return afterReturnProcess(context);
        }
    }

    /**
     * 后置通知
     */
    protected abstract Object afterReturnProcess(TrackContext context);

    /**
     * 异常通知
     */
    protected abstract Object afterThrowingProcess(TrackContext context);


}
